"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const util_1 = require("./util");
class NoteMap {
    constructor() {
        this._reverseMap = new Map();
        this._supportMap = new Map();
        this._instanceMap = new Map();
    }
    static create() {
        return new NoteMap();
    }
    get length() {
        return this._supportMap.size;
    }
    get size() {
        return this._instanceMap.size;
    }
    getAllKeys() {
        return [...this._supportMap.keys()];
    }
    getAllIdentifiers() {
        return [...this._reverseMap.keys()];
    }
    getAllInstances() {
        return [...this._instanceMap.values()];
    }
    keyExists(key) {
        const identifiers = this._getIdentifiers(key);
        return identifiers.length === 0;
    }
    identifierExists(identifier) {
        return this._instanceMap.has(identifier);
    }
    countKey(key) {
        const identifiers = this._getIdentifiers(key);
        return identifiers.length;
    }
    setWithIdentifier(key, identifier, value) {
        this._addSupport(key, identifier);
        this._addInstance(identifier, value);
        return this;
    }
    setAndGetIdentifer(key, value) {
        const identifier = util_1.generateIdentifier();
        this._addSupport(key, identifier);
        this._addInstance(identifier, value);
        return identifier;
    }
    getKeyByIdentifier(identifier) {
        return this._getKey(identifier);
    }
    getInstanceByIdentifier(identifier) {
        return this._getInstance(identifier);
    }
    getAllIdentifiersByKey(key) {
        const identifers = this._getIdentifiers(key);
        return identifers;
    }
    getAllInstancesByKey(key) {
        const identifers = this._getIdentifiers(key);
        return this._getInstances(identifers);
    }
    removeByIdentifier(identifier) {
        const removeIdentifierResult = this._removeIdentifier(identifier);
        if (!removeIdentifierResult) {
            return false;
        }
        return this._removeInstance(identifier);
    }
    removeByValue(value) {
        let count = 0;
        const keys = this.getAllKeys();
        for (const key of keys) {
            const identifiers = this.getAllIdentifiersByKey(key);
            for (const identifier of identifiers) {
                const instance = this.getInstanceByIdentifier(identifier);
                if (instance === value) {
                    count++;
                    this.removeByIdentifier(identifier);
                }
            }
        }
        return count;
    }
    removeAllByKey(key) {
        const identifiers = this._getIdentifiers(key);
        let succeed = 0;
        let failed = 0;
        for (const identifier of identifiers) {
            const result = this.removeByIdentifier(identifier);
            if (result) {
                succeed++;
            }
            else {
                failed++;
            }
        }
        return {
            succeed,
            failed,
        };
    }
    _getKey(identifier) {
        return this._reverseMap.get(identifier);
    }
    _getIdentifiers(key) {
        const identifers = this._supportMap.get(key);
        return (identifers !== null && identifers !== void 0 ? identifers : []);
    }
    _addSupport(key, identifier) {
        this._reverseMap.set(identifier, key);
        if (this._supportMap.has(key)) {
            const list = this._supportMap.get(key);
            const fixedList = (list !== null && list !== void 0 ? list : []);
            this._supportMap.set(key, [...fixedList, identifier]);
        }
        else {
            this._supportMap.set(key, [identifier]);
        }
        return this;
    }
    _removeIdentifier(identifier) {
        const key = this._reverseMap.get(identifier);
        if (!key) {
            return false;
        }
        this._reverseMap.delete(identifier);
        const identifierList = this._supportMap.get(key);
        if (!identifierList) {
            return false;
        }
        const newIdentifierList = identifierList.filter((each) => each !== identifier);
        if (newIdentifierList.length === 0) {
            this._supportMap.delete(key);
        }
        else {
            this._supportMap.set(key, newIdentifierList);
        }
        return true;
    }
    _getInstance(identifer) {
        return this._instanceMap.get(identifer);
    }
    _getInstances(identifers) {
        const result = [];
        for (const identifer of identifers) {
            const value = this._instanceMap.get(identifer);
            if (typeof value !== 'undefined') {
                result.push(value);
            }
        }
        return result;
    }
    _addInstance(identifier, value) {
        this._instanceMap.set(identifier, value);
        return this;
    }
    _removeInstance(identifier) {
        return this._instanceMap.delete(identifier);
    }
}
exports.NoteMap = NoteMap;
